@@ -27,6 +27,7 @@
 
 #include <string.h>
 #include <stdarg.h>
+#include <stdlib.h>
 
 #include "gconf-property-editor.h"
 #include "gconf-property-editor-marshal.h"
@@ -48,8 +49,6 @@ enum {
 	PROP_DATA_FREE_CB
 };
 
-typedef void (*GConfPropertyEditorDataFreeCb) (gpointer data);
-
 struct _GConfPropertyEditorPrivate
 {
 	gchar                   *key;
@@ -61,8 +60,8 @@ struct _GConfPropertyEditorPrivate
 	GConfClientNotifyFunc    callback;
 	gboolean                 inited;
 
-	gpointer 		      data;
-	GConfPropertyEditorDataFreeCb data_free_cb;
+	gpointer 		 data;
+	GFreeFunc                data_free_cb;
 };
 
 typedef struct
@@ -76,13 +75,6 @@ typedef struct
 
 static guint peditor_signals[LAST_SIGNAL];
 
-static GObjectClass *parent_class;
-
-static void gconf_property_editor_init        (GConfPropertyEditor      *gconf_property_editor,
-					       GConfPropertyEditorClass *class);
-static void gconf_property_editor_class_init  (GConfPropertyEditorClass *class);
-static void gconf_property_editor_base_init   (GConfPropertyEditorClass *class);
-
 static void gconf_property_editor_set_prop    (GObject      *object,
 					       guint         prop_id,
 					       const GValue *value,
@@ -103,33 +95,11 @@ static GObject *gconf_peditor_new             (const gchar           *key,
 					       const gchar	     *first_custom,
 					       ...);
 
-GType
-gconf_property_editor_get_type (void)
-{
-	static GType gconf_property_editor_type = 0;
+G_DEFINE_TYPE (GConfPropertyEditor, gconf_property_editor, G_TYPE_OBJECT)
 
-	if (!gconf_property_editor_type) {
-		GTypeInfo gconf_property_editor_info = {
-			sizeof (GConfPropertyEditorClass),
-			(GBaseInitFunc) gconf_property_editor_base_init,
-			NULL, /* GBaseFinalizeFunc */
-			(GClassInitFunc) gconf_property_editor_class_init,
-			NULL, /* GClassFinalizeFunc */
-			NULL, /* user-supplied data */
-			sizeof (GConfPropertyEditor),
-			0, /* n_preallocs */
-			(GInstanceInitFunc) gconf_property_editor_init,
-			NULL
-		};
+#define GCONF_PROPERTY_EDITOR_GET_PRIVATE(object) \
+        (G_TYPE_INSTANCE_GET_PRIVATE ((object), gconf_property_editor_get_type (), GConfPropertyEditorPrivate))
 
-		gconf_property_editor_type =
-			g_type_register_static (G_TYPE_OBJECT,
-						"GConfPropertyEditor",
-						&gconf_property_editor_info, 0);
-	}
-
-	return gconf_property_editor_type;
-}
 
 static GConfValue*
 gconf_property_editor_conv_default (GConfPropertyEditor *peditor,
@@ -139,20 +109,14 @@ gconf_property_editor_conv_default (GConfPropertyEditor *peditor,
 }
 
 static void
-gconf_property_editor_init (GConfPropertyEditor      *gconf_property_editor,
-			    GConfPropertyEditorClass *class)
+gconf_property_editor_init (GConfPropertyEditor *gconf_property_editor)
 {
-	gconf_property_editor->p = g_new0 (GConfPropertyEditorPrivate, 1);
+	gconf_property_editor->p = GCONF_PROPERTY_EDITOR_GET_PRIVATE (gconf_property_editor);
 	gconf_property_editor->p->conv_to_widget_cb = gconf_property_editor_conv_default;
 	gconf_property_editor->p->conv_from_widget_cb = gconf_property_editor_conv_default;
 	gconf_property_editor->p->inited = FALSE;
 }
 
-static void
-gconf_property_editor_base_init (GConfPropertyEditorClass *class)
-{
-}
-
 static void
 gconf_property_editor_class_init (GConfPropertyEditorClass *class)
 {
@@ -216,7 +180,7 @@ gconf_property_editor_class_init (GConfPropertyEditorClass *class)
 		 g_param_spec_pointer ("data",
 				       _("Property editor object data"),
 				       _("Custom data required by the specific property editor"),
-				       G_PARAM_WRITABLE));
+				       G_PARAM_READWRITE));
 
 	g_object_class_install_property
 		(object_class, PROP_DATA_FREE_CB,
@@ -225,8 +189,7 @@ gconf_property_editor_class_init (GConfPropertyEditorClass *class)
 				       _("Callback to be issued when property editor object data is to be freed"),
 				       G_PARAM_WRITABLE));
 
-	parent_class = G_OBJECT_CLASS
-		(g_type_class_ref (G_TYPE_OBJECT));
+	g_type_class_add_private (class, sizeof (GConfPropertyEditorPrivate));
 }
 
 static void
@@ -314,6 +277,9 @@ gconf_property_editor_get_prop (GObject    *object,
 		g_value_set_pointer (value, peditor->p->changeset);
 		break;
 
+	case PROP_DATA:
+		g_value_set_pointer (value, peditor->p->data);
+		break;
 	default:
 		g_warning ("Bad argument get");
 		break;
@@ -344,9 +310,7 @@ gconf_property_editor_finalize (GObject *object)
 		g_object_unref (client);
 	}
 
-	g_free (gconf_property_editor->p);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gconf_property_editor_parent_class)->finalize (object);
 }
 
 static GObject *
@@ -387,8 +351,9 @@ gconf_peditor_new (const gchar           *key,
 	gconf_entry = gconf_client_get_entry (client, GCONF_PROPERTY_EDITOR (obj)->p->key, NULL, TRUE, NULL);
 	GCONF_PROPERTY_EDITOR (obj)->p->callback (client, 0, gconf_entry, obj);
 	GCONF_PROPERTY_EDITOR (obj)->p->inited = TRUE;
-	gconf_entry_free (gconf_entry);
-	g_object_unref (G_OBJECT (client));
+	if (gconf_entry)
+		gconf_entry_free (gconf_entry);
+	g_object_unref (client);
 
 	return obj;
 }
@@ -490,7 +455,7 @@ gconf_peditor_new_boolean (GConfChangeSet *changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (checkbox), "toggled",
+	g_signal_connect_swapped (checkbox, "toggled",
 				  (GCallback) peditor_boolean_widget_changed, peditor);
 
 	return peditor;
@@ -559,7 +524,7 @@ gconf_peditor_new_integer_valist (GConfChangeSet *changeset,
 		 first_property_name,
 		 var_args, NULL);
 
-	g_signal_connect_swapped (G_OBJECT (entry), "changed",
+	g_signal_connect_swapped (entry, "changed",
 				  (GCallback) peditor_integer_widget_changed, peditor);
 
 	return peditor;
@@ -653,7 +618,7 @@ gconf_peditor_new_string_valist (GConfChangeSet *changeset,
 		 first_property_name,
 		 var_args, NULL);
 
-	g_signal_connect_swapped (G_OBJECT (entry), "changed",
+	g_signal_connect_swapped (entry, "changed",
 				  (GCallback) peditor_string_widget_changed, peditor);
 
 	return peditor;
@@ -684,32 +649,6 @@ gconf_peditor_new_string (GConfChangeSet *changeset,
 	return peditor;
 }
 
-GObject *
-gconf_peditor_new_filename (GConfChangeSet *changeset,
-			    const gchar    *key,
-			    GtkWidget      *file_entry,
-			    const gchar    *first_property_name,
-			    ...)
-{
-	GObject *peditor;
-	va_list var_args;
-
-	g_return_val_if_fail (key != NULL, NULL);
-	g_return_val_if_fail (file_entry != NULL, NULL);
-	g_return_val_if_fail (GNOME_IS_FILE_ENTRY (file_entry), NULL);
-
-	va_start (var_args, first_property_name);
-
-	peditor = gconf_peditor_new_string_valist
-		(changeset, key,
-		 gnome_file_entry_gtk_entry (GNOME_FILE_ENTRY (file_entry)),
-		 first_property_name, var_args);
-
-	va_end (var_args);
-
-	return peditor;
-}
-
 static void
 peditor_color_value_changed (GConfClient         *client,
 			     guint                cnxn_id,
@@ -788,7 +727,7 @@ gconf_peditor_new_color (GConfChangeSet *changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (cb), "color_set",
+	g_signal_connect_swapped (cb, "color_set",
 				  (GCallback) peditor_color_widget_changed, peditor);
 
 	return peditor;
@@ -941,7 +880,7 @@ gconf_peditor_new_combo_box (GConfChangeSet *changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (combo_box), "changed",
+	g_signal_connect_swapped (combo_box, "changed",
 				  (GCallback) peditor_combo_box_widget_changed, peditor);
 
 	return peditor;
@@ -991,7 +930,7 @@ gconf_peditor_new_combo_box_with_enum	(GConfChangeSet *changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (combo_box), "changed",
+	g_signal_connect_swapped (combo_box, "changed",
 				  (GCallback) peditor_combo_box_widget_changed, peditor);
 
 	return peditor;
@@ -1078,7 +1017,7 @@ gconf_peditor_new_select_radio (GConfChangeSet *changeset,
 	va_end (var_args);
 
 	for (item = radio_group; item != NULL; item = item->next)
-		g_signal_connect_swapped (G_OBJECT (item->data), "toggled",
+		g_signal_connect_swapped (item->data, "toggled",
 					  (GCallback) peditor_select_radio_widget_changed, peditor);
 
 	return peditor;
@@ -1122,7 +1061,7 @@ peditor_numeric_range_widget_changed (GConfPropertyEditor *peditor,
 	if (!peditor->p->inited) return;
 
 	/* We try to get the default type from the schemas.  if not, we default
-	 * to a float.
+	 * to an int.
 	 */
 	client = gconf_client_get_default();
 
@@ -1131,9 +1070,10 @@ peditor_numeric_range_widget_changed (GConfPropertyEditor *peditor,
 							      NULL);
 	g_object_unref (client);
 
-	if (default_value)
+	if (default_value) {
 		value_wid = gconf_value_new (default_value->type);
-	else {
+		gconf_value_free (default_value);
+	} else {
 		g_warning ("Unable to find a default value for key for %s.\n"
 			   "I'll assume it is an integer, but that may break things.\n"
 			   "Please be sure that the associated schema is installed",
@@ -1141,8 +1081,6 @@ peditor_numeric_range_widget_changed (GConfPropertyEditor *peditor,
 		value_wid = gconf_value_new (GCONF_VALUE_INT);
 	}
 
-	gconf_value_free (default_value);
-
 	g_assert (value_wid);
 
 	if (value_wid->type == GCONF_VALUE_INT)
@@ -1150,7 +1088,7 @@ peditor_numeric_range_widget_changed (GConfPropertyEditor *peditor,
 	else if (value_wid->type == GCONF_VALUE_FLOAT)
 		gconf_value_set_float (value_wid, gtk_adjustment_get_value (adjustment));
 	else {
-		g_warning ("unable to set a gconf key for %s of type %d\n",
+		g_warning ("unable to set a gconf key for %s of type %d",
 			   peditor->p->key,
 			   value_wid->type);
 		gconf_value_free (value_wid);
@@ -1191,7 +1129,7 @@ gconf_peditor_new_numeric_range (GConfChangeSet *changeset,
 		(key,
 		 (GConfClientNotifyFunc) peditor_numeric_range_value_changed,
 		 changeset,
-		 G_OBJECT (adjustment),
+		 adjustment,
 		 first_property_name,
 		 var_args, NULL);
 
@@ -1238,7 +1176,6 @@ gconf_peditor_widget_set_guard (GConfPropertyEditor *peditor,
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
 	client = gconf_client_get_default ();
-
 	value = gconf_client_get (client, peditor->p->key, NULL);
 	g_object_unref (client);
 
@@ -1249,7 +1186,7 @@ gconf_peditor_widget_set_guard (GConfPropertyEditor *peditor,
                 g_warning ("NULL GConf value: %s: possibly incomplete setup", peditor->p->key);
 	}
 
-	g_signal_connect (G_OBJECT (peditor), "value-changed", (GCallback) guard_value_changed, widget);
+	g_signal_connect (peditor, "value-changed", (GCallback) guard_value_changed, widget);
 }
 
 GConfValue *
@@ -1443,7 +1380,7 @@ gconf_peditor_new_enum_toggle  (GConfChangeSet		*changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (checkbox), "toggled",
+	g_signal_connect_swapped (checkbox, "toggled",
 				  (GCallback) peditor_boolean_widget_changed, peditor);
 
 	return peditor;
@@ -1507,6 +1444,7 @@ peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
 						      GTK_DIALOG_MODAL,
 						      GTK_MESSAGE_ERROR,
 						      GTK_BUTTONS_OK,
+						      "%s",
 						      message);
 			gtk_dialog_run (GTK_DIALOG (box));
 			gtk_widget_destroy (box);
@@ -1520,7 +1458,7 @@ peditor_image_set_filename (GConfPropertyEditor *peditor, const gchar *filename)
 	}
 
 	gtk_image_set_from_pixbuf (image, pixbuf);
-	g_object_unref (G_OBJECT (pixbuf));
+	g_object_unref (pixbuf);
 
 	return TRUE;
 }
@@ -1581,7 +1519,7 @@ peditor_image_chooser_update_preview_cb (GtkFileChooser *chooser,
 	g_free (filename);
 
 	if (pixbuf != NULL)
-		gdk_pixbuf_unref (pixbuf);
+		g_object_unref (pixbuf);
 }
 
 static void
@@ -1638,10 +1576,10 @@ peditor_image_clicked_cb (GConfPropertyEditor *peditor, GtkButton *button)
 	if (filename && strcmp (filename, ""))
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser), filename);
 
-	g_signal_connect (G_OBJECT (chooser), "update-preview",
+	g_signal_connect (chooser, "update-preview",
 			  G_CALLBACK (peditor_image_chooser_update_preview_cb),
 			  preview);
-	g_signal_connect (G_OBJECT (chooser), "response",
+	g_signal_connect (chooser, "response",
 			  G_CALLBACK (peditor_image_chooser_response_cb),
 			  peditor);
 
@@ -1701,7 +1639,7 @@ gconf_peditor_new_image (GConfChangeSet	  *changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (button), "clicked",
+	g_signal_connect_swapped (button, "clicked",
 				  (GCallback) peditor_image_clicked_cb, peditor);
 
 	return peditor;
@@ -1755,7 +1693,7 @@ gconf_peditor_new_select_radio_with_enum (GConfChangeSet *changeset,
 	va_end (var_args);
 
 	for (item = radio_group; item != NULL; item = item->next)
-		g_signal_connect_swapped (G_OBJECT (item->data), "toggled",
+		g_signal_connect_swapped (item->data, "toggled",
 					  (GCallback) peditor_select_radio_widget_changed, peditor);
 
 	return peditor;
@@ -1852,7 +1790,7 @@ gconf_peditor_new_tree_view (GConfChangeSet *changeset,
 
 	va_end (var_args);
 
-	g_signal_connect_swapped (G_OBJECT (gtk_tree_view_get_selection (tree_view)),
+	g_signal_connect_swapped (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)),
 				  "changed",
 				  (GCallback) peditor_tree_view_widget_changed, peditor);
 
